Exception: Vivarium::RawStore::FormatError
- Inherits:
-
StandardError
- Object
- StandardError
- Vivarium::RawStore::FormatError
- Defined in:
- lib/vivarium/raw_store.rb
Overview
Raised when a file is not a valid vivarium-raw capture.
Raised when a file is not a valid vivarium-raw capture.